use crate::error::Error;
use r2d2_postgres::postgres::NoTls;
use r2d2_postgres::PostgresConnectionManager;
use std::ops::Deref;
use std::sync::{Arc, Mutex};
use std::time::Duration;

pub enum Pool {
    Postgres(r2d2::Pool<PostgresConnectionManager<NoTls>>),
}

pub enum Client {
    Postgres(r2d2::PooledConnection<PostgresConnectionManager<NoTls>>),
}

static mut POOL: Option<Arc<Mutex<Pool>>> = None;

pub fn init() -> Result<(), Error> {
    let manager = PostgresConnectionManager::new(
        "host=localhost user=postgres dbname=locutus".parse().unwrap(),
        NoTls,
    );

    let pool = r2d2::Pool::builder()
        .max_size(64)
        .min_idle(Some(2))
        .connection_timeout(Duration::from_secs(4))
        .max_lifetime(Some(Duration::from_secs(3600)))
        .build(manager)?;

    unsafe {
        POOL = Some(Arc::new(Mutex::new(Pool::Postgres(pool))));
    }

    Ok(())
}

pub fn client() -> Result<Client, Error> {
    match unsafe { POOL.as_ref().unwrap().clone().lock().unwrap().deref() } {
        Pool::Postgres(pool) => Ok(Client::Postgres(pool.get()?)),
    }
}
