use super::StatusClass;
use super::StatusClass::*;

pub static HTTP_STATUSES: [(u16, StatusClass, &str, &str); 41] = [
    (100, Informational, "Continue",
     "The client SHOULD continue with its request."),
    (101, Informational, "Switching Protocols",
     "The server understands and is willing to comply with the clients request, via the Upgrade message header field, for a change in the application protocol being used on this connection."),

    (200, Success,       "OK",
     "The request has succeeded."),
    (201, Success,       "Created",
     "The request has been fulfilled and resulted in a new resource being created."),
    (202, Success,       "Accepted",
     "The request has been accepted for processing, but the processing has not been completed."),
    (203, Success,       "Non-Authoritative Information",
     "The returned meta information in the entity-header is not the definitive set as available from the origin server, but is gathered from a local or a third-party copy."),
    (204, Success,       "No Content",
     "The server has fulfilled the request but does not need to return an entity-body, and might want to return updated meta information."),
    (205, Success,       "Reset Content",
     "The server has fulfilled the request and the user agent SHOULD reset the document view which caused the request to be sent."),
    (206, Success,       "Partial Content",
     "The server has fulfilled the partial GET request for the resource."),

    (300, Redirection,   "Multiple Choices",
     "The requested resource corresponds to any one of a set of representations, each with its own specific location, and agent-driven negotiation information is being provided so that the user (or user agent) can select a preferred representation and redirect its request to that location."),
    (301, Redirection,   "Moved Permanently",
     "The requested resource has been assigned a new permanent URI and any future references to this resource SHOULD use one of the returned URIs."),
    (302, Redirection,   "Found",
     "The requested resource resides temporarily under a different URI."),
    (303, Redirection,   "See Other",
     "The response to the request can be found under a different URI and SHOULD be retrieved using a GET method on that resource."),
    (304, Success,       "Not Modified",
     "The request has been fulfilled and the requested resource has not been modified."),
    (305, Redirection,   "Use Proxy",
     "The requested resource MUST be accessed through the proxy given by the Location field."),
    (307, Redirection,   "Temporary Redirect",
     "The requested resource resides temporarily under a different URI."),
    (308, Redirection,   "Permanent Redirect",
     "The requested resource has been assigned a new permanent URI and any future references to this resource ought to use one of the enclosed URIs."),

    (400, ClientError,  "Bad Request",
     "The request could not be understood by the server due to malformed syntax."),
    (401, ClientError,  "Unauthorized",
     "The request requires user authentication."),
    (402, ClientError,  "Payment Required",
     ""),
    (403, ClientError,  "Forbidden",
     "The server understood the request, but is refusing to fulfill it."),
    (404, ClientError,  "Not Found",
     "The server has not found anything matching the Request-URI."),
    (405, ClientError,  "Method Not Allowed",
     "The method specified in the Request-Line is not allowed for the resource identified by the Request-URI."),
    (406, ClientError,  "Not Acceptable",
     "The resource identified by the request is only capable of generating response entities which have content characteristics not acceptable according to the accept headers sent in the request."),
    (407, ClientError,  "Proxy Authentication Required",
     "The request requires user authentication on the proxy."),
    (408, ClientError,  "Request Timeout",
     "The client did not produce a request within the time that the server was prepared to wait."),
    (409, ClientError,  "Conflict",
     "The request could not be completed due to a conflict with the current state of the resource."),
    (410, ClientError,  "Gone",
     "The requested resource is no longer available at the server and no forwarding address is known."),
    (411, ClientError,  "Length Required",
     "The server refuses to accept the request without a defined Content-Length."),
    (412, ClientError,  "Precondition Failed",
     "The precondition given in one or more of the request-header fields evaluated to false when it was tested on the server."),
    (413, ClientError,  "Request Entity Too Large",
     "The server is refusing to process a request because the request entity is larger than the server is willing or able to process."),
    (414, ClientError,  "Request-URI Too Long",
     "The server is refusing to service the request because the Request-URI is longer than the server is willing to interpret."),
    (415, ClientError,  "Unsupported Media Type",
     "The server is refusing to service the request because the entity of the request is in a format not supported by the requested resource for the requested method."),
    (416, ClientError,  "Range Not Satisfiable",
     "None of the ranges in the requests Range header field overlap the current extent of the selected resource or that the set of ranges requested has been rejected due to invalid ranges or an excessive request of small or overlapping ranges."),
    (417, ClientError,  "Expectation Failed",
     "The expectation given in an Expect request-header field could not be met by this server, or, if the server is a proxy, the server has unambiguous evidence that the request could not be met by the next-hop server."),

    (500, ServerError,  "Internal Server Error",
     "The server encountered an unexpected condition which prevented it from fulfilling the request." ),
    (501, ServerError,  "Not Implemented",
     "The server does not support the functionality required to fulfill the request."),
    (502, ServerError,  "Bad Gateway",
     "The server, while acting as a gateway or proxy, received an invalid response from the upstream server it accessed in attempting to fulfill the request."),
    (503, ServerError,  "Service Unavailable",
     "The server is currently unable to handle the request due to a temporary overloading or maintenance of the server."),
    (504, ServerError,  "Gateway Timeout",
     "The server, while acting as a gateway or proxy, did not receive a timely response from the upstream server specified by the URI or some other auxiliary server it needed to access in attempting to complete the request."),
    (505, ServerError,  "HTTP Version Not Supported",
     "The server does not support, or refuses to support, the HTTP protocol version that was used in the request message."),
];

pub static DEFAULT_DOCUMENT: &str = "\
    <!DOCTYPE html>\n\
    <html lang=\"en\">\n\
    <head>\n\
    \t<title>{status_code} {status_message} - Locutus - {hostname}</title>\n\
    \t<meta charset=\"UTF-8\"/>\n\
    \t<meta name=\"theme-color\" content=\"{theme_color}\"/>\n\
    \t<meta name=\"color-scheme\" content=\"light dark\"/>\n\
    \t<meta name=\"apple-mobile-web-app-status-bar-style\" content=\"black-translucent\"/>\n\
    \t<meta name=\"viewport\" content=\"width=device-width,initial-scale=1.0\"/>\n\
    \t<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"/favicon.ico\"/>\n\
    \t<style>\n\
    \t\thtml{{font-family:\"Arial\",sans-serif;--error:#C00000;--warning:#E0C000;--success:#008000;--info:#606060;--color:var(--{color_name});}}\n\
    \t\tbody{{background-color:#F0F0F0;margin:0;}}\n\
    \t\tmain{{max-width:650px;margin:2em auto;}}\n\
    \t\tsection{{margin:1em;background-color:#FFFFFF;border: 1px solid var(--color);border-radius:4px;padding:1em;}}\n\
    \t\th1,h2,h3,h4,h5,h6,h7{{text-align:center;color:var(--color);font-weight:normal;}}\n\
    \t\th1{{font-size:3em;margin:0.125em 0 0.125em 0;}}\n\
    \t\th2{{font-size:1.5em;margin:0.25em 0 1em 0;}}\n\
    \t\tp{{text-align:center;font-size:0.875em;}}\n\
    \t\tdiv.footer{{color:#808080;font-size:0.75em;text-align:center;margin:2em 0 0.5em 0;}}\n\
    \t\tdiv.footer a{{color:#808080;}}\n\
    \t\t@media(prefers-color-scheme:dark){{\n\
    \t\t\thtml{{color:#FFFFFF;}}\n\
    \t\t\tbody{{background-color:#101010;}}\n\
    \t\t\tsection{{background-color:#181818;}}\n\
    \t\t}}\n\
    \t</style>\n\
    </head>\n\
    <body>\n\
    \t<main>\n\
    \t\t<section>\n\
    {doc}\
    \t\t\t<div class=\"footer\"><a href=\"https://{hostname}/\">{hostname}</a> - {server_str}</div>\n\
    \t\t</section>\n\
    \t</main>\n\
    </body>\n\
    </html>\n";

pub static ERROR_DOCUMENT: &str = "\
    \t\t\t<h1>{code}</h1>\n\
    \t\t\t<h2>{message} :&#xFEFF;(</h2>\n\
    \t\t\t<p>{desc}</p>\n\
    \t\t\t<p>{info}</p>\n";

pub static WARNING_DOCUMENT: &str = "\
    \t\t\t<h1>{code}</h1>\n\
    \t\t\t<h2>{message} :&#xFEFF;o</h2>\n\
    \t\t\t<p>{desc}</p>\n\
    \t\t\t<p>{info}</p>\n";

pub static SUCCESS_DOCUMENT: &str = "\
    \t\t\t<h1>{code}</h1>\n\
    \t\t\t<h2>{message} :&#xFEFF;)</h2>\n\
    \t\t\t<p>{desc}</p>\n\
    \t\t\t<p>{info}</p>\n";

pub static INFO_DOCUMENT: &str = "\
    \t\t\t<h1>{code}</h1>\n\
    \t\t\t<h2>{message} :&#xFEFF;)</h2>\n\
    \t\t\t<p>{desc}</p>\n\
    \t\t\t<p>{info}</p>\n";
